<?php


namespace CKPL\Pay\Model;


class DisallowHTMLTagsTrimmer
{
    private static $COMMUNICATION_PROTOCOLS_REGEX = '#^[a-z]*?://|.*(www.)#';
    private static $HTML_SPECIAL_REGEX = '/[&#][a-zA-Z0-9]*?;.*?/';
    private static $HTML_BRACKET_REGEX = '/[<>]/';
    private static $HTML_MULTIPLE_SPACE_REGEX = '/\s+/';
    private static $TAB = '\t';

    /**
     * @param string $value
     * @return string
     */
    public static function trim(string $value): string
    {
        $value = self::replaceCommunicationProtocolTags($value);
        $value = self::replaceDefaultHtmlTags($value);
        $value = self::replaceSpecialHtmlTags($value);
        $value = self::replaceBracketSign($value);
        $value = self::replaceTabSign($value);
        $value = self::replaceMultipleSpace($value);
        return $value;
    }

    /**
     * @param string $value
     * @return string
     */
    private static function replaceCommunicationProtocolTags(string $value): string
    {
        return preg_replace(self::$COMMUNICATION_PROTOCOLS_REGEX, '', $value);
    }

    /**
     * @param string $value
     * @return string
     */
    private static function replaceDefaultHtmlTags(string $value): string
    {
        return htmlentities(strip_tags(html_entity_decode($value)));
    }

    /**
     * @param string $value
     * @return string
     */
    private static function replaceSpecialHtmlTags(string $value): string
    {
        return preg_replace(self::$HTML_SPECIAL_REGEX, '', $value);
    }

    /**
     * @param string $value
     * @return string
     */
    private static function replaceBracketSign(string $value): string
    {
        return preg_replace(self::$HTML_BRACKET_REGEX, '', $value);
    }

    /**
     * @param string $value
     * @return string
     */
    private static function replaceTabSign(string $value): string
    {
        return str_replace(self::$TAB, '', $value);
    }

    /**
     * @param string $value
     * @return string
     */
    private static function replaceMultipleSpace(string $value): string
    {
        return preg_replace(self::$HTML_MULTIPLE_SPACE_REGEX, ' ', $value);
    }

}
